/**
 * @file MotionDetection.h
 * @author Hans Haupt
 * @brief This component controls the IMU (Accelerometer & Gyroscope) ICM-42670-P
 * @version 0.1
 * @date 2023-12-15
 * 
 * @copyright Copyright (c) 2023
 * 
 */
#ifndef MotionDetection_h
#define MotionDetection_h
#include <SPI.h>
#include <Arduino.h>
struct IMUResult{
    int16_t x;
    int16_t y;
    int16_t z;
};


class MotionDetection{
protected:
  
    static const uint8_t CMD_READ =      0x80;
    static const uint8_t CMD_WRITE =     0x00;
    static const uint8_t ADDR_MASK =     0x7F;

    //Registers
    static const uint8_t REG_TEMP_LOW =  0x0A;
    static const uint8_t REG_TEMP_HIGH = 0X09;

    static const uint8_t ACCEL_DATA_X_HIGH = 0x0B;
    static const uint8_t ACCEL_DATA_X_LOW =  0x0C;
    static const uint8_t ACCEL_DATA_Y_HIGH = 0x0D;
    static const uint8_t ACCEL_DATA_Y_LOW =  0x0E;
    static const uint8_t ACCEL_DATA_Z_HIGH = 0x0F;
    static const uint8_t ACCEL_DATA_Z_LOW =  0x10;

    static const uint8_t GYRO_DATA_X_HIGH =  0x11;
    static const uint8_t GYRO_DATA_X_LOW =   0x12;
    static const uint8_t GYRO_DATA_Y_HIGH =  0x13;
    static const uint8_t GYRO_DATA_Y_LOW =   0x14;
    static const uint8_t GYRO_DATA_Z_HIGH =  0x15;
    static const uint8_t GYRO_DATA_Z_LOW =   0x16;

    static const uint8_t PWR_MGMT0 =         0x1F;
    static const uint8_t WHO_AM_I =          0x75;

    static const uint frequency = 10000000;

    uint16_t cmdRead(uint8_t regHigh,uint8_t regLow);
    uint16_t cmdWrite(uint8_t regHigh,uint8_t regLow);
    uint8_t cmdRead(uint8_t reg);
    uint8_t cmdWrite(uint8_t reg);

    int8_t readRegister(uint8_t reg);
    int16_t readDoubleRegister(uint8_t lowerReg);

    SPIClass * handler = NULL;
     
    
public:
    MotionDetection();

    /**
     * @brief initialized the  IMU Component. 
     * Wakes the IMU from Standby 
     * Set configuration 
     * 
     */
    void begin(void);

    /**
     * @brief stops the component
     * Sets the IMU to Low-Power-Mode
     * 
     */
    void end(void);

    /**
     * @brief Triggers a new Reading of the accelerationvalues and reads them from the IMU
     * 
     * @return IMUResult that contains the new read values
     */
    IMUResult getAcceleration();

    /**
     * @brief Triggers a new reading of the gyroscope and reads the values from the imu
     * 
     * @return IMUResult 
     */
    IMUResult getRotation();

    /**
     * @brief Reads the current On Chip temperature of the IMU 
     * 
     * @return normalized temperature in degree Centigrade
     */
    float getTemperature();

    /**
     * @brief Returns the value of reading the whoAmI register
     * When IMU working correctly, value should be 0x67
     * 
     * @return the value of the whoami register of the ICM-42670 
     */
    int8_t getWhoAmI();
};
#endif //MotionDetection